Package: KEGGlincs
Type: Package
Title: Visualize all edges within a KEGG pathway and overlay LINCS data
Version: 1.38.0
Date: 2016-06-02
Author: Shana White
Maintainer: Shana White <vandersm@mail.uc.edu>, Mario Medvedovic <medvedm@ucmail.uc.edu>
Description: See what is going on 'under the hood' of KEGG pathways by
        explicitly re-creating the pathway maps from information
        obtained from KGML files.
License: GPL-3
RoxygenNote: 6.1.1
Depends: R (>= 3.3), KOdata, hgu133a.db, org.Hs.eg.db (>= 3.3.0)
SystemRequirements: Cytoscape (>= 3.3.0), Java (>= 8)
Suggests: BiocManager (>= 1.20.3), knitr, graph
biocViews: NetworkInference, GeneExpression, DataRepresentation,
        ThirdPartyClient,CellBiology,GraphAndNetwork,Pathways,KEGG,Network
Imports:
        AnnotationDbi,KEGGgraph,igraph,plyr,gtools,httr,RJSONIO,KEGGREST,
        methods,graphics,stats,utils, XML, grDevices
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/KEGGlincs
git_branch: RELEASE_3_23
git_last_commit: 78ba67f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:52:24 UTC; biocbuild
